@@ -5,6 +5,8 @@ use std::mem;
 
 use crate::Error;
 
+type HtmlString = Vec<u8>;
+
 /// An emitter is an object providing methods to the tokenizer to produce tokens.
 ///
 /// Domain-specific applications of the HTML tokenizer can manually implement this trait to
@@ -36,7 +38,7 @@ pub trait Emitter {
     ///
     /// This is primarily for testing purposes. This is *not* supposed to override the tag name of
     /// the current tag.
-    fn set_last_start_tag(&mut self, last_start_tag: Option<&str>);
+    fn set_last_start_tag(&mut self, last_start_tag: Option<&[u8]>);
 
     /// The state machine has reached the end of the file. It will soon call `pop_token` for the
     /// last time.
@@ -50,7 +52,7 @@ pub trait Emitter {
     fn pop_token(&mut self) -> Option<Self::Token>;
 
     /// Emit a bunch of plain characters as character tokens.
-    fn emit_string(&mut self, c: &str);
+    fn emit_string(&mut self, c: &[u8]);
 
     /// Set the _current token_ to a start tag.
     fn init_start_tag(&mut self);
@@ -97,17 +99,17 @@ pub trait Emitter {
     /// Assuming the _current token_ is a start/end tag, append a string to the current tag's name.
     ///
     /// If the current token is not a start or end tag, this method may panic.
-    fn push_tag_name(&mut self, s: &str);
+    fn push_tag_name(&mut self, s: &[u8]);
 
     /// Assuming the _current token_ is a comment, append a string to the comment's contents.
     ///
     /// If the current token is not a comment, this method may panic.
-    fn push_comment(&mut self, s: &str);
+    fn push_comment(&mut self, s: &[u8]);
 
     /// Assuming the _current token_ is a doctype, append a string to the doctype's name.
     ///
     /// If the current token is not a doctype, this method may panic.
-    fn push_doctype_name(&mut self, s: &str);
+    fn push_doctype_name(&mut self, s: &[u8]);
 
     /// Set the _current token_ to a new doctype token:
     ///
@@ -132,32 +134,32 @@ pub trait Emitter {
     /// Append a string to the current attribute's name.
     ///
     /// If there is no current attribute, this method may panic.
-    fn push_attribute_name(&mut self, s: &str);
+    fn push_attribute_name(&mut self, s: &[u8]);
 
     /// Append a string to the current attribute's value.
     ///
     /// If there is no current attribute, this method may panic.
-    fn push_attribute_value(&mut self, s: &str);
+    fn push_attribute_value(&mut self, s: &[u8]);
 
     /// Assuming the _current token_ is a doctype, set its "public identifier" to the given string.
     ///
     /// If the current token is not a doctype, this method may panic.
-    fn set_doctype_public_identifier(&mut self, value: &str);
+    fn set_doctype_public_identifier(&mut self, value: &[u8]);
 
     /// Assuming the _current token_ is a doctype, set its "system identifier" to the given string.
     ///
     /// If the current token is not a doctype, this method may panic.
-    fn set_doctype_system_identifier(&mut self, value: &str);
+    fn set_doctype_system_identifier(&mut self, value: &[u8]);
 
     /// Assuming the _current token_ is a doctype, append a string to its "public identifier" to the given string.
     ///
     /// If the current token is not a doctype, this method may panic.
-    fn push_doctype_public_identifier(&mut self, s: &str);
+    fn push_doctype_public_identifier(&mut self, s: &[u8]);
 
     /// Assuming the _current token_ is a doctype, append a string to its "system identifier" to the given string.
     ///
     /// If the current token is not a doctype, this method may panic.
-    fn push_doctype_system_identifier(&mut self, s: &str);
+    fn push_doctype_system_identifier(&mut self, s: &[u8]);
 
     /// Return true if all of these hold. Return false otherwise.
     ///
@@ -173,11 +175,11 @@ pub trait Emitter {
 /// The default implementation of [`crate::Emitter`], used to produce ("emit") tokens.
 #[derive(Default)]
 pub struct DefaultEmitter {
-    current_characters: String,
+    current_characters: Vec<u8>,
     current_token: Option<Token>,
-    last_start_tag: String,
-    current_attribute: Option<(String, String)>,
-    seen_attributes: BTreeSet<String>,
+    last_start_tag: Vec<u8>,
+    current_attribute: Option<(HtmlString, HtmlString)>,
+    seen_attributes: BTreeSet<HtmlString>,
     emitted_tokens: VecDeque<Token>,
 }
 
@@ -228,10 +230,10 @@ impl DefaultEmitter {
 impl Emitter for DefaultEmitter {
     type Token = Token;
 
-    fn set_last_start_tag(&mut self, last_start_tag: Option<&str>) {
+    fn set_last_start_tag(&mut self, last_start_tag: Option<&[u8]>) {
         self.last_start_tag.clear();
         self.last_start_tag
-            .push_str(last_start_tag.unwrap_or_default());
+            .extend(last_start_tag.unwrap_or_default());
     }
 
     fn emit_eof(&mut self) {
@@ -248,20 +250,20 @@ impl Emitter for DefaultEmitter {
         self.emitted_tokens.pop_back()
     }
 
-    fn emit_string(&mut self, s: &str) {
-        self.current_characters.push_str(s);
+    fn emit_string(&mut self, s: &[u8]) {
+        self.current_characters.extend(s);
     }
 
     fn init_start_tag(&mut self) {
-        self.current_token = Some(Token::StartTag(Default::default()));
+        self.current_token = Some(Token::StartTag(StartTag::default()));
     }
     fn init_end_tag(&mut self) {
-        self.current_token = Some(Token::EndTag(Default::default()));
+        self.current_token = Some(Token::EndTag(EndTag::default()));
         self.seen_attributes.clear();
     }
 
     fn init_comment(&mut self) {
-        self.current_token = Some(Token::Comment(String::new()));
+        self.current_token = Some(Token::Comment(Vec::new()));
     }
     fn emit_current_tag(&mut self) {
         self.flush_current_attribute();
@@ -273,8 +275,8 @@ impl Emitter for DefaultEmitter {
                 }
                 self.seen_attributes.clear();
             }
-            Token::StartTag(ref mut _tag) => {
-                self.set_last_start_tag(Some(&_tag.name));
+            Token::StartTag(ref mut tag) => {
+                self.set_last_start_tag(Some(&tag.name));
             }
             _ => debug_assert!(false),
         }
@@ -315,34 +317,34 @@ impl Emitter for DefaultEmitter {
             _ => debug_assert!(false),
         }
     }
-    fn push_tag_name(&mut self, s: &str) {
+    fn push_tag_name(&mut self, s: &[u8]) {
         match self.current_token {
-            Some(Token::StartTag(StartTag { ref mut name, .. })) => {
-                name.push_str(s);
-            }
-            Some(Token::EndTag(EndTag { ref mut name, .. })) => {
-                name.push_str(s);
+            Some(
+                Token::StartTag(StartTag { ref mut name, .. })
+                | Token::EndTag(EndTag { ref mut name, .. }),
+            ) => {
+                name.extend(s);
             }
             _ => debug_assert!(false),
         }
     }
 
-    fn push_comment(&mut self, s: &str) {
+    fn push_comment(&mut self, s: &[u8]) {
         match self.current_token {
-            Some(Token::Comment(ref mut data)) => data.push_str(s),
+            Some(Token::Comment(ref mut data)) => data.extend(s),
             _ => debug_assert!(false),
         }
     }
 
-    fn push_doctype_name(&mut self, s: &str) {
+    fn push_doctype_name(&mut self, s: &[u8]) {
         match self.current_token {
-            Some(Token::Doctype(ref mut doctype)) => doctype.name.push_str(s),
+            Some(Token::Doctype(ref mut doctype)) => doctype.name.extend(s),
             _ => debug_assert!(false),
         }
     }
     fn init_doctype(&mut self) {
         self.current_token = Some(Token::Doctype(Doctype {
-            name: String::new(),
+            name: Vec::new(),
             force_quirks: false,
             public_identifier: None,
             system_identifier: None,
@@ -351,54 +353,54 @@ impl Emitter for DefaultEmitter {
 
     fn init_attribute(&mut self) {
         self.flush_current_attribute();
-        self.current_attribute = Some((String::new(), String::new()));
+        self.current_attribute = Some((Vec::new(), Vec::new()));
     }
-    fn push_attribute_name(&mut self, s: &str) {
-        self.current_attribute.as_mut().unwrap().0.push_str(s);
+    fn push_attribute_name(&mut self, s: &[u8]) {
+        self.current_attribute.as_mut().unwrap().0.extend(s);
     }
-    fn push_attribute_value(&mut self, s: &str) {
-        self.current_attribute.as_mut().unwrap().1.push_str(s);
+    fn push_attribute_value(&mut self, s: &[u8]) {
+        self.current_attribute.as_mut().unwrap().1.extend(s);
     }
-    fn set_doctype_public_identifier(&mut self, value: &str) {
+    fn set_doctype_public_identifier(&mut self, value: &[u8]) {
         if let Some(Token::Doctype(Doctype {
             ref mut public_identifier,
             ..
         })) = self.current_token
         {
-            *public_identifier = Some(value.to_owned());
+            *public_identifier = Some(value.to_vec());
         } else {
             debug_assert!(false);
         }
     }
-    fn set_doctype_system_identifier(&mut self, value: &str) {
+    fn set_doctype_system_identifier(&mut self, value: &[u8]) {
         if let Some(Token::Doctype(Doctype {
             ref mut system_identifier,
             ..
         })) = self.current_token
         {
-            *system_identifier = Some(value.to_owned());
+            *system_identifier = Some(value.to_vec());
         } else {
             debug_assert!(false);
         }
     }
-    fn push_doctype_public_identifier(&mut self, s: &str) {
+    fn push_doctype_public_identifier(&mut self, s: &[u8]) {
         if let Some(Token::Doctype(Doctype {
             public_identifier: Some(ref mut id),
             ..
         })) = self.current_token
         {
-            id.push_str(s);
+            id.extend(s);
         } else {
             debug_assert!(false);
         }
     }
-    fn push_doctype_system_identifier(&mut self, s: &str) {
+    fn push_doctype_system_identifier(&mut self, s: &[u8]) {
         if let Some(Token::Doctype(Doctype {
             system_identifier: Some(ref mut id),
             ..
         })) = self.current_token
         {
-            id.push_str(s);
+            id.extend(s);
         } else {
             debug_assert!(false);
         }
@@ -415,27 +417,27 @@ impl Emitter for DefaultEmitter {
 }
 
 /// A HTML end/close tag, such as `<p>` or `<a>`.
-#[derive(Debug, Default, Eq, PartialEq)]
+#[derive(Debug, Default, Eq, PartialEq, Clone)]
 pub struct StartTag {
     /// Whether this tag is self-closing. If it is self-closing, no following [`EndTag`] should be
     /// expected.
     pub self_closing: bool,
 
     /// The start tag's name, such as `"p"` or `"a"`.
-    pub name: String,
+    pub name: HtmlString,
 
     /// A mapping for any HTML attributes this start tag may have.
     ///
     /// Duplicate attributes are ignored after the first one as per WHATWG spec. Implement your own
     /// [`Emitter`] to tweak this behavior.
-    pub attributes: BTreeMap<String, String>,
+    pub attributes: BTreeMap<HtmlString, HtmlString>,
 }
 
 /// A HTML end/close tag, such as `</p>` or `</a>`.
-#[derive(Debug, Default, Eq, PartialEq)]
+#[derive(Debug, Default, Eq, PartialEq, Clone)]
 pub struct EndTag {
     /// The ending tag's name, such as `"p"` or `"a"`.
-    pub name: String,
+    pub name: HtmlString,
 }
 
 /// A doctype. Some examples:
@@ -444,33 +446,33 @@ pub struct EndTag {
 /// * `<!DOCTYPE {name} PUBLIC '{public_identifier}'>`
 /// * `<!DOCTYPE {name} SYSTEM '{system_identifier}'>`
 /// * `<!DOCTYPE {name} PUBLIC '{public_identifier}' '{system_identifier}'>`
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Clone)]
 pub struct Doctype {
     /// The ["force quirks"](https://html.spec.whatwg.org/#force-quirks-flag) flag.
     pub force_quirks: bool,
 
     /// The doctype's name. For HTML documents this is "html".
-    pub name: String,
+    pub name: HtmlString,
 
     /// The doctype's public identifier.
-    pub public_identifier: Option<String>,
+    pub public_identifier: Option<HtmlString>,
 
     /// The doctype's system identifier.
-    pub system_identifier: Option<String>,
+    pub system_identifier: Option<HtmlString>,
 }
 
 /// The token type used by default. You can define your own token type by implementing the
 /// [`crate::Emitter`] trait and using [`crate::Tokenizer::new_with_emitter`].
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Clone)]
 pub enum Token {
     /// A HTML start tag.
     StartTag(StartTag),
     /// A HTML end tag.
     EndTag(EndTag),
     /// A literal string.
-    String(String),
+    String(HtmlString),
     /// A HTML comment.
-    Comment(String),
+    Comment(HtmlString),
     /// A HTML doctype declaration.
     Doctype(Doctype),
     /// A HTML parsing error.
