@@ -1,12 +1,16 @@
+use std::convert::Infallible;
+
+use crate::char_validator::CharValidator;
 use crate::machine;
 use crate::machine_helper::MachineHelper;
 use crate::read_helper::ReadHelper;
 use crate::utils::{ControlToken, State};
-use crate::{DefaultEmitter, Emitter, Never, Readable, Reader};
+use crate::{DefaultEmitter, Emitter, Readable, Reader};
 
 /// A HTML tokenizer. See crate-level docs for basic usage.
 pub struct Tokenizer<R: Reader, E: Emitter = DefaultEmitter> {
     eof: bool,
+    pub(crate) validator: CharValidator,
     pub(crate) emitter: E,
     pub(crate) reader: ReadHelper<R>,
     pub(crate) machine_helper: MachineHelper,
@@ -33,6 +37,7 @@ impl<R: Reader, E: Emitter> Tokenizer<R, E> {
     pub fn new_with_emitter<'a, S: Readable<'a, Reader = R>>(input: S, emitter: E) -> Self {
         Tokenizer {
             eof: false,
+            validator: CharValidator::default(),
             emitter,
             reader: ReadHelper::new(input.to_reader()),
             machine_helper: MachineHelper::default(),
@@ -60,7 +65,8 @@ impl<R: Reader, E: Emitter> Tokenizer<R, E> {
     /// Only available with the `integration-tests` feature which is not public API.
     #[cfg(feature = "integration-tests")]
     pub fn set_last_start_tag(&mut self, last_start_tag: Option<&str>) {
-        self.emitter.set_last_start_tag(last_start_tag);
+        self.emitter
+            .set_last_start_tag(last_start_tag.map(str::as_bytes));
     }
 }
 
@@ -75,6 +81,7 @@ impl<R: Reader, E: Emitter> Iterator for Tokenizer<R, E> {
                 match machine::consume(self) {
                     Ok(ControlToken::Continue) => (),
                     Ok(ControlToken::Eof) => {
+                        self.validator.flush_character_error(&mut self.emitter);
                         self.eof = true;
                         self.emitter.emit_eof();
                     }
@@ -91,9 +98,9 @@ impl<R: Reader, E: Emitter> Iterator for Tokenizer<R, E> {
 /// `Result<Token, _>`.
 ///
 /// This is the return value of [`Tokenizer::infallible`].
-pub struct InfallibleTokenizer<R: Reader<Error = Never>, E: Emitter>(Tokenizer<R, E>);
+pub struct InfallibleTokenizer<R: Reader<Error = Infallible>, E: Emitter>(Tokenizer<R, E>);
 
-impl<R: Reader<Error = Never>, E: Emitter> Tokenizer<R, E> {
+impl<R: Reader<Error = Infallible>, E: Emitter> Tokenizer<R, E> {
     /// Statically assert that this iterator is infallible.
     ///
     /// Call this to get rid of error handling when parsing HTML from strings.
@@ -102,7 +109,7 @@ impl<R: Reader<Error = Never>, E: Emitter> Tokenizer<R, E> {
     }
 }
 
-impl<R: Reader<Error = Never>, E: Emitter> Iterator for InfallibleTokenizer<R, E> {
+impl<R: Reader<Error = Infallible>, E: Emitter> Iterator for InfallibleTokenizer<R, E> {
     type Item = E::Token;
 
     fn next(&mut self) -> Option<Self::Item> {
