@@ -6,28 +6,6 @@ macro_rules! surrogate_pat {
 
 pub(crate) use surrogate_pat;
 
-macro_rules! control_pat {
-    () => (0x0d | 0x0000..=0x001f | 0x007f..=0x009f)
-}
-
-pub(crate) use control_pat;
-
-macro_rules! ascii_digit_pat {
-    () => {
-        '0'..='9'
-    };
-}
-
-pub(crate) use ascii_digit_pat;
-
-macro_rules! whitespace_pat {
-    () => {
-        '\t' | '\u{0A}' | '\u{0C}' | ' '
-    };
-}
-
-pub(crate) use whitespace_pat;
-
 macro_rules! noncharacter_pat {
     () => {
         0xfdd0
@@ -152,7 +130,6 @@ pub enum State {
     AmbiguousAmpersand,
     NumericCharacterReference,
     HexadecimalCharacterReferenceStart,
-    DecimalCharacterReferenceStart,
     HexadecimalCharacterReference,
     DecimalCharacterReference,
     NumericCharacterReferenceEnd,
@@ -172,12 +149,31 @@ macro_rules! ctostr {
 pub(crate) use ctostr;
 
 /// Repeatedly call `f` with chunks of lowercased characters from `s`.
-pub(crate) fn with_lowercase_str(s: &str, mut f: impl FnMut(&str)) {
-    if s.chars().any(|x| x.is_ascii_uppercase()) {
-        for x in s.chars() {
-            f(ctostr!(x.to_ascii_lowercase()))
+pub(crate) fn with_lowercase_str(s: &[u8], mut f: impl FnMut(&[u8])) {
+    if s.iter().any(u8::is_ascii_uppercase) {
+        for x in s {
+            f(&[x.to_ascii_lowercase()]);
         }
     } else {
         f(s);
     }
 }
+
+// having this be a macro is performance critical. rustc appears to be unable to optimize away code
+// like this:
+//
+// ```rust
+// fn noop(s: &str) {}
+//
+// noop(&format!("foo"));
+// ```
+//
+// format!() + its string allocation still exists in resulting code
+macro_rules! trace_log {
+    ($($tt:tt)*) => {{
+        #[cfg(feature = "integration-tests")]
+        crate::testutils::trace_log(&format!($($tt)*));
+    }};
+}
+
+pub(crate) use trace_log;
